package com.develop.common.widget

import android.content.Context
import android.graphics.Rect
import android.util.AttributeSet
import android.view.View
import android.widget.ImageView
import android.widget.LinearLayout
import androidx.appcompat.widget.AppCompatTextView
import androidx.recyclerview.widget.RecyclerView
import coil.load
import com.develop.base.ext.resId2Dimension
import com.develop.base.ext.updateText
import com.develop.common.R
import com.develop.common.data_repo.db.NewsModel
import com.develop.common.databinding.NewsViewBinding
import com.drake.brv.utils.linear
import com.drake.brv.utils.models
import com.drake.brv.utils.setup

class NewsLayout : LinearLayout {
    private var binding: NewsViewBinding
    private var dataList = mutableListOf<NewsModel>()

    constructor(context: Context?) : super(context) {}
    constructor(context: Context?, attrs: AttributeSet?) : super(context, attrs) {}
    constructor(context: Context?, attrs: AttributeSet?, defStyleAttr: Int) : super(
        context, attrs, defStyleAttr
    ) {
    }

    var onNewsBackClickListener: OnNewsNewsBackClickListener? = null

    init {
        val root = View.inflate(context, R.layout.news_view, this)
        binding = NewsViewBinding.bind(root)
        binding.newRecycler.apply {
            linear(RecyclerView.VERTICAL)
            setup {
                addType<NewsModel>(R.layout.item_news)
                onBind {
                    val model = getModel<NewsModel>()
                    val ivCircle = findView<ImageView>(R.id.iv_circle)
                    val tvNewsTitle = findView<AppCompatTextView>(R.id.tv_news_title)
                    if (model.hasRead) {
                        ivCircle.load(R.drawable.b1b2b2_circle)
                        tvNewsTitle.setTextColor(resources.getColor(R.color.color_B1B2B2))
                    } else {
                        ivCircle.load(R.drawable.e60012_circle)
                        tvNewsTitle.setTextColor(resources.getColor(R.color.color_E60012))
                    }
                    tvNewsTitle.updateText(model.des)
                    findView<AppCompatTextView>(R.id.tv_news_content).updateText(model.des)
                }
            }
            addItemDecoration(object : RecyclerView.ItemDecoration() {
                override fun getItemOffsets(
                    outRect: Rect, view: View, parent: RecyclerView, state: RecyclerView.State
                ) {
                    super.getItemOffsets(outRect, view, parent, state)
                    outRect.left = dp44
                    outRect.right = dp44
                    if (parent.getChildAdapterPosition(view) == 0) {
                        outRect.top = dp57
                    }
                    outRect.bottom = dp60
                }
            })

        }.models = dataList
        binding.topBar.setTopBarTitle(context.getString(R.string.notice))
        binding.topBar.listenBackClick {
            onNewsBackClickListener?.onNewsBackClick()
        }
    }


    fun updateNewsData(data: MutableList<NewsModel>) {
        dataList.clear()
        dataList.addAll(data)
        binding.newRecycler.models = dataList
    }

    interface OnNewsNewsBackClickListener {
        fun onNewsBackClick()
    }

    companion object {
        var dp44 = (com.develop.base.R.dimen.convert_44px).resId2Dimension().toInt()
        var dp57 = (com.develop.base.R.dimen.convert_57px).resId2Dimension().toInt()
        var dp60 = (com.develop.base.R.dimen.convert_60px).resId2Dimension().toInt()
    }

}