package com.develop.setting.ui

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.widget.ImageView
import android.widget.RelativeLayout
import androidx.appcompat.widget.AppCompatTextView
import androidx.cardview.widget.CardView
import coil.load
import com.alibaba.android.arouter.facade.annotation.Route
import com.blankj.utilcode.util.LanguageUtils
import com.develop.base.ext.navigateTo
import com.develop.base.ext.resId2Dimension
import com.develop.base.ext.updateText
import com.develop.base.mvvm.BaseBindingActivity
import com.develop.base.util.MMkvUtils
import com.develop.common.data_repo.db.DataFactory
import com.develop.common.data_repo.db.LanguageModel
import com.develop.common.event.FinishAtyEvent
import com.develop.common.router.Screens
import com.develop.common.tag.*
import com.develop.common.dialog.CancelConfirmDialog
import com.develop.setting.R
import com.develop.setting.databinding.ActivityLanguageBinding
import com.drake.brv.utils.bindingAdapter
import com.drake.brv.utils.linear
import com.drake.brv.utils.models
import com.drake.brv.utils.setup
import com.kuyuntech.cofarcooking.device.sdk.constant.core.DevStatus
import com.kuyuntech.cofarcooking.device.sdk.util.core.CofarSDK
import org.greenrobot.eventbus.EventBus
import org.greenrobot.eventbus.Subscribe
import java.util.*

@Route(path = Screens.Setting.LANGUAGE)
class LanguageActivity : BaseBindingActivity<ActivityLanguageBinding>() {
    private val cancelConfirmDialog by lazy {
        CancelConfirmDialog()
    }
    private var selectedPosition = -1
    private var isLanguageTip = false

    override fun createViewBinding(inflater: LayoutInflater): ActivityLanguageBinding {
        return ActivityLanguageBinding.inflate(inflater)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        EventBus.getDefault().register(this)
        val isFromSetting = intent.extras?.getBoolean(LANGUAGE_FROM_SETTING) ?: false
        if (isFromSetting) {
            binding.apply {
                ivBanner.visibility = View.GONE
                topBar.visibility = View.VISIBLE
                topBar.setTopBarTitle(getString(com.develop.common.R.string.language))
                topBar.listenBackClick {
                    finish()
                }
                tvLanguage.visibility = View.GONE
                val params = languageRecycle.layoutParams as RelativeLayout.LayoutParams
                params.topMargin = dp251
                languageRecycle.layoutParams = params
                tvNext.updateText(getString(com.develop.common.R.string.confirm))
            }
        }

        binding.languageRecycle.apply {
            linear()
            setup {
                addType<LanguageModel>(R.layout.item_language_view)
                onBind {
                    val ivIcon = findView<ImageView>(R.id.ic_icon)
                    val tvTitle = findView<AppCompatTextView>(R.id.tv_title)
                    val cardView = findView<CardView>(R.id.cardView)
                    if (selectedPosition == absoluteAdapterPosition) {
                        tvTitle.setTextColor(context.resources.getColor(com.develop.common.R.color.color_F4A12E))
                        cardView.setCardBackgroundColor(context.resources.getColor(com.develop.common.R.color.color_33F4A12E))
                        cardView.cardElevation = 0.1f
                    } else {
                        tvTitle.setTextColor(context.resources.getColor(com.develop.common.R.color.color_6B6B6B))
                        cardView.setCardBackgroundColor(context.resources.getColor(com.develop.common.R.color.color_fff))
                        cardView.cardElevation = dp6
                    }
                    val model = getModel<LanguageModel>()
                    ivIcon.load(model.icon)
                    tvTitle.updateText(model.languageName)
                }
                R.id.cardView.onClick {
                    updateSelectionPos(absoluteAdapterPosition)
                }
            }
        }.models = DataFactory.createLanguageData(resources)

        selectedPosition = getSelectedPos()

        cancelConfirmDialog.onDialogClickListener =
            object : CancelConfirmDialog.OnDialogClickListener {
                override fun onConfirm() {
                    if (isLanguageTip) {
                        updateLanguage()
                        navigateTo(Screens.Main.ENTRANCE_CHOSEN)
                        finish()
                    }
                }

                override fun onCancel() {

                }

            }

        binding.tvNext.setOnClickListener {
            if (CofarSDK.devInfo().status == DevStatus.RUNNING.toInt()) {
                isLanguageTip = false
                cancelConfirmDialog.title = getString(com.develop.common.R.string.change_lang_tips)
                cancelConfirmDialog.apply {
                    showCancel = false
                    showDialog(supportFragmentManager, "confirm")
                }
                return@setOnClickListener
            }
            if (isFromSetting) {
                isLanguageTip = true
                cancelConfirmDialog.title = getString(com.develop.common.R.string.change_lang_tips2)
                cancelConfirmDialog.showDialog(supportFragmentManager, "tips2")
            } else {
                updateLanguage()
                navigateTo(Screens.Setting.POLICY)
            }
        }

    }

    private fun updateSelectionPos(pos: Int) {
        val tempPosition = selectedPosition
        selectedPosition = pos
        binding.languageRecycle.bindingAdapter.apply {
            notifyItemChanged(tempPosition)
            notifyItemChanged(selectedPosition)
        }
    }

    private fun updateLanguage() {
        when (selectedPosition) {
            0, -1 -> {
                MMkvUtils.save(CURRENT_LANGUAGE, EN)
                LanguageUtils.applyLanguage(Locale.ENGLISH)
            }
            1 -> {
                MMkvUtils.save(CURRENT_LANGUAGE, ZH)
                LanguageUtils.applyLanguage(Locale.CHINESE)
            }
            2 -> {
                MMkvUtils.save(CURRENT_LANGUAGE, FR)
                LanguageUtils.applyLanguage(Locale.FRANCE)
            }
            3 -> {
                MMkvUtils.save(CURRENT_LANGUAGE, JA)
                LanguageUtils.applyLanguage(Locale.JAPAN)
            }
        }
    }

    private fun getSelectedPos(): Int {
        return when (MMkvUtils.getString(CURRENT_LANGUAGE)) {
            EN -> {
                updateSelectionPos(0)
                0
            }
            ZH -> {
                updateSelectionPos(1)
                1
            }

            FR -> {
                updateSelectionPos(2)
                2
            }

            JA -> {
                updateSelectionPos(3)
                3
            }
            else -> {
                updateSelectionPos(0)
                0
            }
        }
    }

    @Subscribe
    fun onFinishAtyEvent(event: FinishAtyEvent) {
        finish()
    }

    override fun onDestroy() {
        super.onDestroy()
        EventBus.getDefault().unregister(this)
        cancelConfirmDialog.removeSelf()
    }

    companion object {
        var dp251 = (com.develop.base.R.dimen.convert_251px).resId2Dimension().toInt()
        var dp6 = (com.develop.base.R.dimen.convert_6px).resId2Dimension()
    }

}